@@ -104,6 +104,7 @@ struct _CcDisplayPanel
   AdwComboRow    *primary_display_row;
   AdwPreferencesGroup *single_display_settings_group;
 
+  GtkShortcutController *toplevel_shortcuts;
   GtkShortcut *escape_shortcut;
 
   GSettings           *display_settings;
@@ -360,14 +361,14 @@ monitor_labeler_show (CcDisplayPanel *self)
                              g_variant_new_int32 (number));
     }
 
+  g_variant_builder_close (&builder);
+
   if (number < 2)
     {
       g_variant_builder_clear (&builder);
       return monitor_labeler_hide (self);
     }
 
-  g_variant_builder_close (&builder);
-
   g_dbus_proxy_call (self->shell_proxy,
                      "ShowMonitorLabels",
                      g_variant_builder_end (&builder),
@@ -405,6 +406,8 @@ dialog_toplevel_is_active_changed (CcDisplayPanel *self)
 static void
 reset_titlebar (CcDisplayPanel *self)
 {
+  gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (self->toplevel_shortcuts),
+                                              GTK_PHASE_NONE);
   self->showing_apply_titlebar = FALSE;
   g_object_notify (G_OBJECT (self), "showing-apply-titlebar");
 }
@@ -615,6 +618,7 @@ cc_display_panel_class_init (CcDisplayPanelClass *klass)
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, night_light_row);
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, primary_display_row);
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, single_display_settings_group);
+  gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, toplevel_shortcuts);
 
   gtk_widget_class_bind_template_callback (widget_class, apply_current_configuration);
   gtk_widget_class_bind_template_callback (widget_class, cancel_current_configuration);
@@ -735,7 +739,7 @@ move_display_settings_to_separate_page (CcDisplayPanel *self)
 static void
 rebuild_ui (CcDisplayPanel *self)
 {
-  guint n_outputs, n_active_outputs, n_usable_outputs;
+  guint n_active_outputs, n_usable_outputs;
   GList *outputs, *l;
   CcDisplayConfigType type;
 
@@ -801,7 +805,6 @@ rebuild_ui (CcDisplayPanel *self)
   /* Sync the rebuild lists/buttons */
   set_current_output (self, self->current_output, TRUE);
 
-  n_outputs = g_list_length (outputs);
   type = config_get_current_type (self);
 
   if (n_usable_outputs > 1)
@@ -936,6 +939,9 @@ show_apply_titlebar (CcDisplayPanel *self, gboolean is_applicable)
                                   _("This could be due to hardware limitations."));
     }
 
+  gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (self->toplevel_shortcuts),
+                                              GTK_PHASE_BUBBLE);
+
   self->showing_apply_titlebar = TRUE;
   g_object_notify (G_OBJECT (self), "showing-apply-titlebar");
 }
