@@ -130,7 +130,7 @@ network_combo_changed_cb (NetDeviceMobile *self)
         g_debug ("try to switch to connection %s", object_path);
         connection = (NMConnection*) nm_client_get_connection_by_path (self->client, object_path);
         if (connection != NULL) {
-                nm_device_disconnect (self->device, NULL, NULL);
+                nm_device_disconnect_async (self->device, NULL, NULL, NULL);
                 nm_client_activate_connection_async (self->client,
                                                      connection,
                                                      self->device, NULL, NULL,
@@ -354,7 +354,7 @@ nm_device_mobile_refresh_ui (NetDeviceMobile *self)
         const gchar *route4_text = NULL, *route6_text = NULL;
 
         /* set up the device on/off switch */
-        gtk_widget_show (GTK_WIDGET (self->device_off_switch));
+        gtk_widget_set_visible (GTK_WIDGET (self->device_off_switch), TRUE);
         mobilebb_enabled_toggled (self);
 
         /* set device state, with status */
@@ -403,10 +403,10 @@ nm_device_mobile_refresh_ui (NetDeviceMobile *self)
 
                 route4_text = nm_ip_config_get_gateway (ipv4_config);
         } else {
-                gtk_widget_hide (GTK_WIDGET (self->ipv4_heading_label));
-                gtk_widget_hide (GTK_WIDGET (self->ipv4_label));
-                gtk_widget_hide (GTK_WIDGET (self->dns4_heading_label));
-                gtk_widget_hide (GTK_WIDGET (self->dns4_label));
+                gtk_widget_set_visible (GTK_WIDGET (self->ipv4_heading_label), FALSE);
+                gtk_widget_set_visible (GTK_WIDGET (self->ipv4_label), FALSE);
+                gtk_widget_set_visible (GTK_WIDGET (self->dns4_heading_label), FALSE);
+                gtk_widget_set_visible (GTK_WIDGET (self->dns4_label), FALSE);
         }
 
         ipv6_config = nm_device_get_ip6_config (self->device);
@@ -431,10 +431,10 @@ nm_device_mobile_refresh_ui (NetDeviceMobile *self)
 
                 route6_text =  nm_ip_config_get_gateway (ipv6_config);
         } else {
-                gtk_widget_hide (GTK_WIDGET (self->ipv6_heading_label));
-                gtk_widget_hide (GTK_WIDGET (self->ipv6_label));
-                gtk_widget_hide (GTK_WIDGET (self->dns6_heading_label));
-                gtk_widget_hide (GTK_WIDGET (self->dns6_label));
+                gtk_widget_set_visible (GTK_WIDGET (self->ipv6_heading_label), FALSE);
+                gtk_widget_set_visible (GTK_WIDGET (self->ipv6_label), FALSE);
+                gtk_widget_set_visible (GTK_WIDGET (self->dns6_heading_label), FALSE);
+                gtk_widget_set_visible (GTK_WIDGET (self->dns6_label), FALSE);
         }
 
         if (have_ipv4_address && have_ipv6_address) {
@@ -469,8 +469,8 @@ nm_device_mobile_refresh_ui (NetDeviceMobile *self)
                 gtk_widget_set_valign (GTK_WIDGET (self->route_heading_label), GTK_ALIGN_START);
                 gtk_widget_set_visible (GTK_WIDGET (self->route_label), routes_text != NULL);
         } else {
-                gtk_widget_hide (GTK_WIDGET (self->route_heading_label));
-                gtk_widget_hide (GTK_WIDGET (self->route_label));
+                gtk_widget_set_visible (GTK_WIDGET (self->route_heading_label), FALSE);
+                gtk_widget_set_visible (GTK_WIDGET (self->route_label), FALSE);
         }
 }
 
@@ -504,7 +504,7 @@ device_off_switch_changed_cb (NetDeviceMobile *self)
                 for (i = 0; acs && i < acs->len; i++) {
                         a = (NMActiveConnection*)acs->pdata[i];
                         if (strcmp (nm_active_connection_get_uuid (a), uuid) == 0) {
-                                nm_client_deactivate_connection (self->client, a, NULL, NULL);
+                                nm_client_deactivate_connection_async (self->client, a, NULL, NULL, NULL);
                                 break;
                         }
                 }
