@@ -32,6 +32,8 @@ struct _CcLocationPage
 {
   AdwNavigationPage parent_instance;
 
+  GtkLabel     *privacy_policy_link;
+
   GtkListBox   *location_apps_list_box;
   AdwSwitchRow *location_row;
 
@@ -42,7 +44,6 @@ struct _CcLocationPage
   GVariant     *location_apps_perms;
   GVariant     *location_apps_data;
   GHashTable   *location_app_switches;
-  GHashTable   *location_app_rows;
 
   GtkSizeGroup *location_icon_size_group;
 };
@@ -196,6 +197,7 @@ add_location_app (CcLocationPage *self,
 
   icon = g_app_info_get_icon (G_APP_INFO (app_info));
   w = gtk_image_new_from_gicon (icon);
+  gtk_image_set_icon_size (GTK_IMAGE (w), GTK_ICON_SIZE_LARGE);
   gtk_widget_set_valign (w, GTK_ALIGN_CENTER);
   gtk_size_group_add_widget (self->location_icon_size_group, w);
   adw_action_row_add_prefix (ADW_ACTION_ROW (row), w);
@@ -217,6 +219,7 @@ add_location_app (CcLocationPage *self,
   gtk_switch_set_active (GTK_SWITCH (w), enabled);
   gtk_widget_set_valign (w, GTK_ALIGN_CENTER);
   adw_action_row_add_suffix (ADW_ACTION_ROW (row), w);
+  adw_action_row_set_activatable_widget (ADW_ACTION_ROW (row), w);
 
   g_settings_bind_with_mapping (self->location_settings,
                                 LOCATION_ENABLED,
@@ -231,9 +234,6 @@ add_location_app (CcLocationPage *self,
   g_hash_table_insert (self->location_app_switches,
                        g_strdup (app_id),
                        g_object_ref (w));
-  g_hash_table_insert (self->location_app_rows,
-                       g_strdup (app_id),
-                       g_object_ref (row));
 
   data = g_slice_new (LocationAppStateData);
   data->self = self;
@@ -257,8 +257,6 @@ update_perm_store (CcLocationPage *self,
   GVariantIter iter;
   const gchar *key;
   gchar **value;
-  GHashTableIter row_iter;
-  GtkWidget *row;
 
   g_clear_pointer (&self->location_apps_perms, g_variant_unref);
   self->location_apps_perms = permissions;
@@ -266,19 +264,6 @@ update_perm_store (CcLocationPage *self,
   g_clear_pointer (&self->location_apps_data, g_variant_unref);
   self->location_apps_data = permissions_data;
 
-  /* We iterate over all rows, if the permissions do not contain the app id of
-     the row, we remove it. */
-  g_hash_table_iter_init (&row_iter, self->location_app_rows);
-  while (g_hash_table_iter_next (&row_iter, (gpointer *) &key, (gpointer *) &row))
-    {
-      if (!g_variant_lookup_value (permissions, key, NULL))
-        {
-          gtk_list_box_remove (self->location_apps_list_box, row);
-          g_hash_table_remove (self->location_app_switches, key);
-          g_hash_table_iter_remove (&row_iter);
-        }
-    }
-
   g_variant_iter_init (&iter, permissions);
   while (g_variant_iter_loop (&iter, "{&s^a&s}", &key, &value))
     {
@@ -306,22 +291,12 @@ on_perm_store_signal (GDBusProxy *proxy,
                       gpointer    user_data)
 {
   GVariant *permissions, *permissions_data;
-  g_autoptr(GVariant) boxed_permission_data = NULL;
-  g_autoptr(GVariant) table = NULL;
-  g_autoptr(GVariant) id = NULL;
 
   if (g_strcmp0 (signal_name, "Changed") != 0)
     return;
 
-  table = g_variant_get_child_value (parameters, 0);
-  id = g_variant_get_child_value (parameters, 1);
-
-  if (g_strcmp0 (g_variant_get_string (table, NULL), "location") != 0 || g_strcmp0 (g_variant_get_string (id, NULL), "location") != 0)
-    return;
-
   permissions = g_variant_get_child_value (parameters, 4);
-  boxed_permission_data = g_variant_get_child_value (parameters, 3);
-  permissions_data = g_variant_get_variant (boxed_permission_data);
+  permissions_data = g_variant_get_child_value (parameters, 3);
   update_perm_store (user_data, permissions, permissions_data);
 }
 
@@ -331,7 +306,6 @@ on_perm_store_lookup_done(GObject *source_object,
                           gpointer user_data)
 {
   g_autoptr(GError) error = NULL;
-  g_autoptr(GVariant) boxed_permission_data = NULL;
   GVariant *ret, *permissions, *permissions_data;
 
   ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object),
@@ -346,8 +320,7 @@ on_perm_store_lookup_done(GObject *source_object,
     }
 
   permissions = g_variant_get_child_value (ret, 0);
-  boxed_permission_data = g_variant_get_child_value (ret, 1);
-  permissions_data = g_variant_get_variant (boxed_permission_data);
+  permissions_data = g_variant_get_child_value (ret, 1);
   update_perm_store (user_data, permissions, permissions_data);
 
   g_signal_connect_object (source_object,
@@ -406,7 +379,6 @@ cc_location_page_finalize (GObject *object)
   g_clear_pointer (&self->location_apps_perms, g_variant_unref);
   g_clear_pointer (&self->location_apps_data, g_variant_unref);
   g_clear_pointer (&self->location_app_switches, g_hash_table_unref);
-  g_clear_pointer (&self->location_app_rows, g_hash_table_unref);
 
   G_OBJECT_CLASS (cc_location_page_parent_class)->finalize (object);
 }
@@ -421,6 +393,7 @@ cc_location_page_class_init (CcLocationPageClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/privacy/cc-location-page.ui");
 
+  gtk_widget_class_bind_template_child (widget_class, CcLocationPage, privacy_policy_link);
   gtk_widget_class_bind_template_child (widget_class, CcLocationPage, location_apps_list_box);
   gtk_widget_class_bind_template_child (widget_class, CcLocationPage, location_row);
 }
@@ -428,6 +401,8 @@ cc_location_page_class_init (CcLocationPageClass *klass)
 static void
 cc_location_page_init (CcLocationPage *self)
 {
+  g_autofree gchar *privacy_policy_link = NULL;
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
   self->location_icon_size_group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
@@ -435,6 +410,10 @@ cc_location_page_init (CcLocationPage *self)
 
   self->cancellable = g_cancellable_new ();
 
+  /* Translators: This will be presented as the text of a link to the privacy policy */
+  privacy_policy_link = g_strdup_printf ("<a href='https://location.services.mozilla.com/privacy'>%s</a>", _("Learn about what data is collected, and how it is used."));
+  gtk_label_set_label (GTK_LABEL (self->privacy_policy_link), privacy_policy_link);
+
   g_settings_bind (self->location_settings,
                    LOCATION_ENABLED,
                    self->location_row,
@@ -445,10 +424,6 @@ cc_location_page_init (CcLocationPage *self)
                                                        g_str_equal,
                                                        g_free,
                                                        g_object_unref);
-  self->location_app_rows = g_hash_table_new_full (g_str_hash,
-                                                   g_str_equal,
-                                                   g_free,
-                                                   g_object_unref);
 
   g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
                             G_DBUS_PROXY_FLAGS_NONE,
