@@ -51,12 +51,14 @@ cc_privacy_panel_get_help_uri (CcPanel *panel)
   AdwNavigationPage *page = adw_navigation_view_get_visible_page (CC_PRIVACY_PANEL (panel)->navigation);
   const char *page_tag = adw_navigation_page_get_tag (page);
 
-  if (g_strcmp0 (page_tag, "location") == 0)
-    return "help:gnome-help/privacy-location";
-  else if (g_strcmp0 (page_tag, "screenlock") == 0)
-    return "help:gnome-help/privacy-screen-lock";
+  if (g_strcmp0 (page_tag, "camera-page") == 0)
+    return "help:gnome-help/camera";
+  else if (g_strcmp0 (page_tag, "location-page") == 0)
+    return "help:gnome-help/location";
+  else if (g_strcmp0 (page_tag, "microphone-page") == 0)
+    return "help:gnome-help/microphone";
   else
-    return "help:gnome-help/privacy";
+    return NULL;
 }
 
 static void
@@ -81,6 +83,21 @@ cc_privacy_panel_class_init (CcPrivacyPanelClass *klass)
   g_type_ensure (CC_TYPE_USAGE_PAGE);
 }
 
+static void
+on_subpage_set (CcPrivacyPanel *self)
+{
+  AdwNavigationPage *subpage;
+  g_autofree gchar *tag = NULL;
+
+  g_object_get (self, "subpage", &tag, NULL);
+  if (!tag)
+    return;
+
+  subpage = adw_navigation_view_find_page (self->navigation, tag);
+  if (subpage)
+    adw_navigation_view_push (self->navigation, subpage);
+}
+
 static void
 cc_privacy_panel_init (CcPrivacyPanel *self)
 {
@@ -96,4 +113,6 @@ cc_privacy_panel_init (CcPrivacyPanel *self)
   g_object_bind_property (bolt_page, "visible",
                           self->bolt_row, "visible", G_BINDING_SYNC_CREATE);
 #endif
+
+  g_signal_connect_object (self, "notify::subpage", G_CALLBACK (on_subpage_set), self, G_CONNECT_SWAPPED);
 }
