@@ -23,14 +23,14 @@
 
 struct _CcCursorSizeDialog
 {
-  GtkDialog parent;
+  AdwWindow parent;
 
-  GtkGrid *size_grid;
+  GtkBox *cursor_box;
 
   GSettings *interface_settings;
 };
 
-G_DEFINE_TYPE (CcCursorSizeDialog, cc_cursor_size_dialog, GTK_TYPE_DIALOG);
+G_DEFINE_TYPE (CcCursorSizeDialog, cc_cursor_size_dialog, ADW_TYPE_WINDOW);
 
 static void
 cursor_size_toggled (CcCursorSizeDialog *self, GtkWidget *button)
@@ -67,7 +67,7 @@ cc_cursor_size_dialog_class_init (CcCursorSizeDialogClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/universal-access/cc-cursor-size-dialog.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, CcCursorSizeDialog, size_grid);
+  gtk_widget_class_bind_template_child (widget_class, CcCursorSizeDialog, cursor_box);
 }
 
 static void
@@ -75,7 +75,6 @@ cc_cursor_size_dialog_init (CcCursorSizeDialog *self)
 {
   guint cursor_sizes[] = { 24, 32, 48, 64, 96 };
   guint current_cursor_size, i;
-  GtkSizeGroup *size_group;
   GtkWidget *last_radio_button = NULL;
 
   gtk_widget_init_template (GTK_WIDGET (self));
@@ -84,7 +83,6 @@ cc_cursor_size_dialog_init (CcCursorSizeDialog *self)
 
   current_cursor_size = g_settings_get_int (self->interface_settings,
                                             KEY_MOUSE_CURSOR_SIZE);
-  size_group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
 
   for (i = 0; i < G_N_ELEMENTS(cursor_sizes); i++)
     {
@@ -92,10 +90,8 @@ cc_cursor_size_dialog_init (CcCursorSizeDialog *self)
       g_autofree gchar *cursor_image_name = NULL;
 
       cursor_image_name = g_strdup_printf ("/org/gnome/control-center/universal-access/left_ptr_%dpx.png", cursor_sizes[i]);
-      image = gtk_image_new_from_resource (cursor_image_name);
-      gtk_image_set_pixel_size (GTK_IMAGE (image), cursor_sizes[i]);
-      gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
-      gtk_widget_set_valign (image, GTK_ALIGN_CENTER);
+      image = gtk_picture_new_for_resource (cursor_image_name);
+      gtk_picture_set_content_fit (GTK_PICTURE (image), GTK_CONTENT_FIT_SCALE_DOWN);
 
       button = gtk_toggle_button_new ();
       gtk_toggle_button_set_group (GTK_TOGGLE_BUTTON (button), GTK_TOGGLE_BUTTON (last_radio_button));
@@ -103,8 +99,7 @@ cc_cursor_size_dialog_init (CcCursorSizeDialog *self)
       g_object_set_data (G_OBJECT (button), "cursor-size", GUINT_TO_POINTER (cursor_sizes[i]));
 
       gtk_button_set_child (GTK_BUTTON (button), image);
-      gtk_grid_attach (GTK_GRID (self->size_grid), button, i, 0, 1, 1);
-      gtk_size_group_add_widget (size_group, button);
+      gtk_box_append (self->cursor_box, button);
 
       g_signal_connect_object (button, "toggled",
                                G_CALLBACK (cursor_size_toggled), self, G_CONNECT_SWAPPED);
@@ -117,7 +112,5 @@ cc_cursor_size_dialog_init (CcCursorSizeDialog *self)
 CcCursorSizeDialog *
 cc_cursor_size_dialog_new (void)
 {
-  return g_object_new (cc_cursor_size_dialog_get_type (),
-                       "use-header-bar", TRUE,
-                       NULL);
+  return g_object_new (cc_cursor_size_dialog_get_type (), NULL);
 }
