@@ -18,10 +18,9 @@
  * Author: Thomas Wood <thos@gnome.org>
  */
 
-#ifndef _CC_WINDOW_H
-#define _CC_WINDOW_H
+#pragma once
 
-#include <glib-object.h>
+#include <adwaita.h>
 #include "cc-shell.h"
 #include "cc-shell-model.h"
 
@@ -29,7 +28,7 @@ G_BEGIN_DECLS
 
 #define CC_TYPE_WINDOW (cc_window_get_type ())
 
-G_DECLARE_FINAL_TYPE (CcWindow, cc_window, CC, WINDOW, GtkApplicationWindow)
+G_DECLARE_FINAL_TYPE (CcWindow, cc_window, CC, WINDOW, AdwApplicationWindow)
 
 CcWindow *cc_window_new (GtkApplication *application,
                          CcShellModel   *model);
@@ -38,5 +37,3 @@ void cc_window_set_search_item (CcWindow *center,
                                 const char *search);
 
 G_END_DECLS
-
-#endif /* _CC_WINDOW_H */
